package net.sf.doolin.template.support;

import net.sf.doolin.template.TemplateEngine;
import net.sf.doolin.template.TemplateManager;
import net.sf.doolin.template.TemplateMessages;
import net.sf.jstring.LocalizableException;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import javax.annotation.PostConstruct;
import java.util.*;

/**
 * Utility root ancestor for template manager implementations.
 * 
 * @author Damien Coraboeuf
 */
public abstract class AbstractTemplateManager implements TemplateManager {

	/**
	 * Logger
	 */
	private final static Logger log = LoggerFactory
			.getLogger(AbstractTemplateManager.class);

	/**
	 * Indexed list of engines
	 */
	private Map<String, TemplateEngine> engines = new HashMap<String, TemplateEngine>();

	/**
	 * Returns the indexed list of engines
	 * 
	 * @return Indexed list of engines
	 */
	public Map<String, TemplateEngine> getEngines() {
		return this.engines;
	}

	/**
	 * @see net.sf.doolin.template.TemplateManager#getTemplateEngine(java.lang.String)
	 */
	public TemplateEngine getTemplateEngine(String engineName) {
		TemplateEngine engine = this.engines.get(engineName);
		if (engine != null) {
			return engine;
		} else {
			throw new LocalizableException(TemplateMessages.CANNOT_FIND_ENGINE,
					engineName);
		}
	}

	/**
	 * @see net.sf.doolin.template.TemplateManager#getTemplateEngineNames()
	 */
	public List<String> getTemplateEngineNames() {
		return Collections.unmodifiableList(new ArrayList<String>(this.engines
				.keySet()));
	}

	/**
	 * Initialisation of engines.
	 */
	@PostConstruct
	public void postConstruct() {
		// Initialization of the engines
		for (TemplateEngine engine : this.engines.values()) {
			log.info("Initializing engine " + engine);
			engine.init();
		}
	}

	/**
	 * Registers an engine.
	 * 
	 * @param engineName
	 *            Engine name
	 * @param engine
	 *            Engine to register
	 */
	protected void registerEngine(String engineName, TemplateEngine engine) {
		this.engines.put(engineName, engine);
	}

	/**
	 * Sets the indexed list of engines
	 * 
	 * @param engines
	 *            Indexed list of engines
	 */
	public void setEngines(Map<String, TemplateEngine> engines) {
		this.engines = engines;
	}

}
